----------------------------------------------------------------------------------
--   Attribute handler, this contains functions made to mess with attributes	--
--		it also contains stance switching handler code			--
----------------------------------------------------------------------------------
local Attributes =  FlexBar2:NewModule("Attributes", "LibRockEvent-1.0");
-- If this var is incremented, SetDefaults() will be called for this module thus resetting all its settings
Attributes.Version = 1;

-- Get an instance of this lovely SecureStateDriverFrame for use with stance, shift & stealth changes
local SecureStateDriverFrame = CreateFrame("Frame", nil, UIParent, "SecureStateDriverTemplate");
Attributes.SecureStateDriverFrame = SecureStateDriverFrame;
-- Function called after FlexBar2's OnInitialize by Ace2
function Attributes:Activate()
	SecureStateDriverFrame:SetAttribute("statemap-form", "$input");
	self:AddEventListener("FlexBar2_EventManager", "SpellCacheUpdate", "GenerateStateButton");
	self:GenerateStateDriverMagic();
end

-- declare all shapeshift stuff, quite complicated func
function Attributes:GenerateStateDriverMagic()
	-- 'magic' code passed to the state driver to make our state changes happen, typically this is something like [stance:0]0;[stance1:1,stealth:1]1;... depending on your class
	local StateDriverMagic = "";
	-- Loop through the shapeshifts in order to declare the StateDriverMagic, active_statebutton & inactive-statebutton var, note, we do nothing for state 0, its added as a fallback state at the end. Also declare statebutton attributes
	-- Also need to support heal & harm remaps
	for State = 1, GetNumShapeshiftForms() do
		-- Get name of current form
		local _ , FormName = GetShapeshiftFormInfo(State);
		FormName = FormName:gsub(" ", "");
		-- Add a stealth mode for catform
		if(Class == "DRUID" and FormName == "CatForm") then
			-- Redunant double assigning, but it makes the code more readable
			StateDriverMagic = StateDriverMagic .. "[help,stance:" .. State .. ",stealth:1]Stealth-help;[help,stance:" .. State .. ",stealth:Unshifted-help]" .. FormName .. ";";
			StateDriverMagic = StateDriverMagic .. "[harm,stance:" .. State .. ",stealth:1]Stealth-harm;[harm,stance:" .. State .. ",stealth:Unshifted-harm]" .. FormName .. ";";
			StateDriverMagic = StateDriverMagic .. "[stance:" .. State .. ",stealth:1]Stealth-none;[stance:" .. State .. ",stealth:Unshifted-none]" .. FormName .. "-none;";

		else
			-- Redunant double assigning, but it makes the code more readable
			StateDriverMagic = StateDriverMagic .. "[help,stance:" .. State .. "]" .. FormName .. "-help;";
			StateDriverMagic = StateDriverMagic .. "[harm,stance:" .. State .. "]" .. FormName .. "-harm;";
			StateDriverMagic = StateDriverMagic .. "[stance:" .. State .. "]" .. FormName .. "-none;";
		end
	end
	-- Add the unshifted state
	StateDriverMagic = StateDriverMagic .. 	"[help]" .. "Unshifted-help;";
	StateDriverMagic = StateDriverMagic .. 	"[harm]" .. "Unshifted-harm;";
	StateDriverMagic = StateDriverMagic .. 	"Unshifted-none;";

	RegisterStateDriver(SecureStateDriverFrame, "form", StateDriverMagic);
	-- Update all buttons after this has been done
	FlexBar2:Debug("state driver magic declared: ", StateDriverMagic);
	Attributes.StateDriverMagic = StateDriverMagic;
end

-- Declare the button mixin
Attributes.ButtonMixin = {};

-- During initialization add this button to the state driver's list of buttons to manage
function Attributes.ButtonMixin:Load()
	SecureStateDriverFrame:SetAttribute("addchild", self.Frame);
	self.AttributeList = {};
	self.AttributeCache = {};
end

local function ScheduleAttributeRecache(self) self.RecacheAttributes = true; end;
function Attributes.ButtonMixin:Activate()
	self.ModifierPrefix = self:GetModifierPrefix();
	self.ModifierSuffix = self:GetModifierSuffix();
	self:RegisterScript("OnAttributeChanged", ScheduleAttributeRecache);
	self:RegisterScript("OnAttributeChanged", "CheckModifierAttributes");
	self:AddEventListener("PLAYER_TARGET_CHANGED", "CheckModifierAttributes");
	self:AddEventListener("MODIFIER_STATE_CHANGED", "CheckModifierAttributes");
	-- Cache the current prefix and suffix into OldPrefix & OldSuffix respectivly for use by CheckModifiedAttribute (which on its turn triggers with OnAttributeChanged)
	end

function Attributes.ButtonMixin:Deactivate()
	self:UnregisterScript("OnAttributeCache", ScheduleAttributeRecache);
	self:UnregisterScript("OnAttributeChanged", "CheckModifierAttributes");
	self:RemoveEventListener("PLAYER_TARGET_CHANGED", "CheckModifierAttributes");
	self:RemoveEventListener("MODIFIER_STATE_CHANGED", "CheckModifierAttributes");
	self:ClearAllAttributes();
end


-- Sets state changing to false by default and all attributes to nil
function Attributes.ButtonMixin:SetDefaults()
    self.InfoTable.Attributes = {};
	self.InfoTable.StateButton = false;
end

-- Load saved settings
function Attributes.ButtonMixin:LoadSettings()
	for Attribute, Value in pairs(self.InfoTable.Attributes) do
		self:SetAttribute(Attribute, Value, false);
	end
	self:GenerateStateButton();
end

function Attributes.ButtonMixin:SaveAttributeName(Trigger, Frame, Attribute)
	self.AttributeList[Attribute] = true;
end

function Attributes.ButtonMixin:ClearAllAttributes()
	for Attribute, _  in pairs(self.AttributeList) do
		self:SetAttribute(Attribute, "", false);
		self.AttributeCache[Attribute] = nil;
	end
end


-- Generate the state button attribute for this button, this is done on the fly to avoid complications with various combinations
function Attributes.ButtonMixin:GenerateStateButton()
	local RemapShapeshift = self.InfoTable.RemapShapeshift;
	local RemapTarget = self.InfoTable.RemapTarget;
	local StateButton = "";
	local _, Class = UnitClass("player");
	for State = 0, GetNumShapeshiftForms() do
		local FormName, StateName;
		if(State == 0) then
			FormName = "Unshifted";
		else
			_ , FormName = GetShapeshiftFormInfo(State);
			FormName = FormName:gsub(" ", "");
		end
		-- We'll suppose for now that the result from GetShapeshiftFormInfo is not localized
		if(Class == "DRUID" and FormName == "CatForm") then
			FormName = "Stealth";
		end
		if(RemapShapeshift) then
			StateName = FormName;
		else
			StateName = "Unshifted";
		end
		-- Add found form to the state list
		StateButton = StateButton .. FormName .. "-help:" .. StateName .. (RemapTarget and "-help" or "-none") .. ";"
		StateButton = StateButton .. FormName .. "-harm:" .. StateName .. (RemapTarget and "-harm" or "-none") .. ";"
		StateButton = StateButton .. FormName .. "-none:" .. StateName .. "-none;"
	end
	self:SetAttribute("statebutton", StateButton, false);
end

function Attributes.ButtonMixin:GetModifierPrefix()
	return self.ModifierPrefix or SecureButton_GetModifierPrefix();
end

function Attributes.ButtonMixin:GetModifierSuffix()
	return self.ModifierSuffix or SecureButton_GetButtonSuffix(SecureStateChild_GetEffectiveButton(self.Frame));
end


function Attributes.ButtonMixin:CheckModifierAttributes()
	local OldPrefix = self.ModifierPrefix or "";
	local OldSuffix = self.ModifierSuffix or "";
	local NewPrefix = SecureButton_GetModifierPrefix();
	local NewSuffix = SecureButton_GetButtonSuffix(SecureStateChild_GetEffectiveButton(self.Frame));
	if(OldPrefix ~= NewPrefix or OldSuffix ~= NewSuffix) then
		-- Update the prefix & suffix cache then trigger an event that a ModifiedAttribute has changed
		self.ModifierPrefix = NewPrefix;
		self.ModifierSuffix = NewSuffix;
		self:DispatchEvent("ModifierAttributeChanged", self, OldPrefix, OldSuffix, NewPrefix, NewSuffix);
	end
end

-- Gets an attribute on the button, usable both in & out of combat, but not very usefull in combat as SetAttribute is not avail
function Attributes.ButtonMixin:GetAttribute(Attribute)
	local AttributeCache = self.AttributeCache;
	if(self.RecacheAttributes == true) then
		for Key in pairs(AttributeCache) do
			AttributeCache[Key] = nil;
		end
	end
	return AttributeCache[Attribute] or self.Frame:GetAttribute(Attribute);
end

-- This will call :GetAttribute with the button prefix (ctrl-alt-shift-) before it and the button suffix (the current state) after it. if nothing is found with a certain prefix * is used as prefix
function Attributes.ButtonMixin:GetModifiedAttribute(Attribute)
	if(Attribute) then
		return self:GetAttribute(self:GetModifierPrefix() .. Attribute .. self:GetModifierSuffix()) or 
		self:GetAttribute("*" .. Attribute .. self:GetModifierSuffix());
	end
end

-- Set an attribute on the frame, not usable in combat
function Attributes.ButtonMixin:SetAttribute(Attribute, Value, Save)
	self.Frame:SetAttribute(Attribute, Value);
	if(Save ~= false) then self.InfoTable.Attributes[Attribute] = Value; end
	FlexBar2:Debug("Setting Attribute [", Attribute, "] to ", Value);
	-- Put the attribute in the cache so it can be removed if needed
	if(Value ~= "") then self.AttributeList[Attribute] = true; end
	self.AttributeCache[Attribute] = Value;
end

-- Same as :GetAttribute except it sets the attribute instead of getting it :)
function Attributes.ButtonMixin:SetModifiedAttribute(Attribute, Value, Save)
	local Prefix = self:GetModifierPrefix();
	if(Prefix == "") then Prefix = "*"; end
	self:SetAttribute(Prefix .. Attribute .. self:GetModifierSuffix(), Value, Save); 
end
